# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import

from .api import API


class Stats(API):
    _endpoint = 'stats'

    def get(self, table='', start=0, limit=0, order=None, where=None):
        """
        Get a list of stat items

        :param table: str database table name
        :param start: int
        :param limit: int
        :param order: list|tuple
        :param where: list|tuple
        :return:
        """
        parameters = {}

        args = ['start', 'limit', 'order', 'where']
        for arg in args:
            if arg in locals() and locals()[arg]:
                parameters[arg] = locals()[arg]

        response = self._client.session.get(
            '{url}/{table}'.format(
                url=self.endpoint_url, table=table
            ),
            params=parameters
        )
        return self.process_response(response)

    def delete(self, obj_id):
        return self.action_not_supported('delete')

    def get_list(
        self,
        search='',
        start=0,
        limit=0,
        order_by='',
        order_by_dir='ASC',
        published_only=False,
        minimal=False
    ):
        return self.action_not_supported('get_list')

    def create(self, parameters):
        return self.action_not_supported('create')

    def get_published_list(
        self, search='', start=0, limit=0, order_by='', order_by_dir='ASC'
    ):
        return self.action_not_supported('get_published_list')

    def edit(self, obj_id, parameters, create_if_not_exists=False):
        return self.action_not_supported('edit')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import

from .api import API


class Users(API):
    _endpoint = 'users'

    def get_self(self):
        """
        Get your (API) user

        :return: dict|str
        """
        response = self._client.session.get(
            '{url}/self'.format(url=self.endpoint_url)
        )
        return self.process_response(response)

    def check_permission(self, obj_id, permissions):
        """
        Get list of permissions for a user

        :param obj_id: int
        :param permissions: str|list|tuple
        :return: dict|str
        """
        response = self._client.session.post(
            '{url}/{id}/permissioncheck'.format(
                url=self.endpoint_url, id=obj_id
            ),
            data={'permissions': permissions}
        )
        return self.process_response(response)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import

from .api import API


class Contacts(API):
    # Contact unsubscribed themselves.
    UNSUBSCRIBED = 1
    # Contact was unsubscribed due to an unsuccessful send.
    BOUNCED = 2
    # Contact was manually unsubscribed by user.
    MANUAL = 3

    _endpoint = 'contacts'

    def get_owners(self):
        """
        Get a list of users available as contact owners

        :return: dict|str
        """
        response = self._client.session.get(
            '{url}/list/owners'.format(url=self.endpoint_url)
        )
        return self.process_response(response)

    def get_field_list(self):
        """
        Get a list of custom fields

        :return: dict|str
        """
        response = self._client.session.get(
            '{url}/list/fields'.format(url=self.endpoint_url)
        )
        return self.process_response(response)

    def get_segments(self):
        """
        Get a list of contact segments

        :return: dict|str
        """
        response = self._client.session.get(
            '{url}/list/segments'.format(url=self.endpoint_url)
        )
        return self.process_response(response)

    def get_events(
        self,
        obj_id,
        search='',
        include_events=None,
        exclude_events=None,
        order_by='',
        order_by_dir='ASC',
        page=1
    ):
        """
        Get a list of a contact's engagement events

        :param obj_id: int Contact ID
        :param search: str
        :param include_events: list|tuple
        :param exclude_events: list|tuple
        :param order_by: str
        :param order_by_dir: str
        :param page: int
        :return: dict|str
        """
        if include_events is None:
            include_events = []
        if exclude_events is None:
            exclude_events = []

        parameters = {
            'search': search,
            'includeEvents': include_events,
            'excludeEvents': exclude_events,
            'orderBy': order_by,
            'orderByDir': order_by_dir,
            'page': page
        }
        response = self._client.session.get(
            '{url}/{id}/events'.format(
                url=self.endpoint_url, id=obj_id
            ),
            params=parameters
        )
        return self.process_response(response)

    def get_contact_notes(
        self,
        obj_id,
        search='',
        start=0,
        limit=0,
        order_by='',
        order_by_dir='ASC'
    ):
        """
        Get a list of a contact's notes

        :param obj_id: int Contact ID
        :param search: str
        :param start: int
        :param limit: int
        :param order_by: str
        :param order_by_dir: str
        :return: dict|str
        """

        parameters = {
            'search': search,
            'start': start,
            'limit': limit,
            'orderBy': order_by,
            'orderByDir': order_by_dir,
        }
        response = self._client.session.get(
            '{url}/{id}/notes'.format(
                url=self.endpoint_url, id=obj_id
            ),
            params=parameters
        )
        return self.process_response(response)

    def get_contact_segments(self, obj_id):
        """
        Get a segment of smart segments the contact is in

        :param obj_id: int
        :return: dict|str
        """

        response = self._client.session.get(
            '{url}/{id}/segments'.format(
                url=self.endpoint_url, id=obj_id
            )
        )
        return self.process_response(response)

    def get_contact_campaigns(self, obj_id):
        """
        Get a segment of campaigns the contact is in

        :param obj_id: int
        :return: dict|str
        """

        response = self._client.session.get(
            '{url}/{id}/campaigns'.format(
                url=self.endpoint_url, id=obj_id
            )
        )
        return self.process_response(response)

    def add_points(self, obj_id, points, **kwargs):
        """
        Add the points to a contact

        :param obj_id: int
        :param points: int
        :param kwargs: dict 'eventname' and 'actionname'
        :return: dict|str
        """

        response = self._client.session.post(
            '{url}/{id}/points/plus/{points}'.format(
                url=self.endpoint_url, id=obj_id, points=points
            ),
            data=kwargs
        )
        return self.process_response(response)

    def subtract_points(self, obj_id, points, **kwargs):
        """
        Subtract points from a contact

        :param obj_id: int
        :param points: int
        :param kwargs: dict 'eventname' and 'actionname'
        :return: dict|str
        """

        response = self._client.session.post(
            '{url}/{id}/points/minus/{points}'.format(
                url=self.endpoint_url, id=obj_id, points=points
            ),
            data=kwargs
        )
        return self.process_response(response)

    def add_dnc(
        self,
        obj_id,
        channel='email',
        reason=MANUAL,
        channel_id=None,
        comments='via API'
    ):
        """
        Adds Do Not Contact

        :param obj_id: int
        :param channel: str
        :param reason: str
        :param channel_id: int
        :param comments: str
        :return: dict|str
        """
        data = {
            'reason': reason,
            'channelId': channel_id,
            'comments': comments
        }
        response = self._client.session.post(
            '{url}/{id}/dnc/add/{channel}'.format(
                url=self.endpoint_url, id=obj_id, channel=channel
            ),
            data=data
        )
        return self.process_response(response)

    def remove_dnc(self, obj_id, channel):
        """
        Removes Do Not Contact

        :param obj_id: int
        :param channel: str
        :return: dict|str
        """
        response = self._client.session.post(
            '{url}/{id}/dnc/remove/{channel}'.format(
                url=self.endpoint_url, id=obj_id, channel=channel
            )
        )
        return self.process_response(response)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import

from .api import API


class Points(API):
    _endpoint = 'points'

    def get_point_action_types(self):
        response = self._client.session.get(
            '{url}/actions/types'.format(url=self.endpoint_url)
        )
        return self.process_response(response)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import

from .api import API


class PointTriggers(API):
    _endpoint = 'points/triggers'

    def delete_trigger_events(self, trigger_id, event_ids):
        """
        Remove events from a point trigger

        :param trigger_id: int
        :param event_ids: list|tuple
        :return: dict|str
        """

        response = self._client.session.delete(
            '{url}/{trigger_id}/events/delete'.format(
                url=self.endpoint_url, trigger_id=trigger_id
            ),
            params={'events': event_ids}
        )
        return self.process_response(response)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import

from .api import API


class Data(API):
    _endpoint = 'data'

    def get(self, data_type, options=None):
        """
        Get a single item
        :param data_type: str
        :param options: dict
        :return: dict|str
        """
        if options is None:
            options = {}
        response = self._client.session.get(
            '{url}/{type}'.format(
                url=self.endpoint_url, type=data_type
            ),
            params=options
        )
        return self.process_response(response)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import


class MauticException(Exception):
    """
    Base class for Mautic exceptions.
    """
    code = 500
    default_message = 'Mautic error occurred.'

    def __init__(self, message=None):
        if message is not None:
            self.message = message
        else:
            self.message = self.default_message

    def __repr__(self):
        return '{} {}'.format(
            type(self).__name__,
            self.message,
        )

    def __str__(self):
        return self.__repr__()


class ActionNotSupportedException(MauticException):
    """
    Exception representing an unsupported action
    """

    default_message = 'Action is not supported at this time.'


class ContextNotFoundException(MauticException):
    """
    Exception representing a requested API context which was not found
    """

    default_message = 'Context not found.'


class IncorrectParametersReturnedException(MauticException):
    """
    Exception representing an incorrect parameter set for an OAuth token request
    """

    default_message = 'Incorrect parameters returned.'


class UnexpectedResponseFormatException(MauticException):
    """
    Exception representing an unexpected HTTP response
    """

    default_message = 'The response returned is in an unexpected format.'



# -*- coding: utf-8 -*-
from .exceptions import (
    ActionNotSupportedException,
    ContextNotFoundException,
    MauticException,
    UnexpectedResponseFormatException,
)
from .api import MauticOauth2Client, MauticBasicAuthClient
from .assets import Assets
from .campaigns import Campaigns
from .categories import Categories
from .companies import Companies
from .company_fields import CompanyFields
from .contacts import Contacts
from .contact_fields import ContactFields
from .contacts import Contacts
from .data import Data
from .dynamic_contents import DynamicContents
from .emails import Emails
from .files import Files
from .forms import Forms
from .notes import Notes
from .notifications import Notifications
from .pages import Pages
from .point_triggers import PointTriggers
from .points import Points
from .reports import Reports
from .roles import Roles
from .segments import Segments
from .smses import Smses
from .stages import Stages
from .stats import Stats
from .users import Users

__version__ = '0.2.0'



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import

from .api import API


class Reports(API):
    _endpoint = 'reports'

    def create(self, parameters):
        return self.action_not_supported('create')

    def edit(self, obj_id, parameters, create_if_not_exists=False):
        return self.action_not_supported('edit')

    def delete(self, obj_id):
        return self.action_not_supported('delete')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import

from .api import API


class Stages(API):
    _endpoint = 'stages'

    def add_contact(self, obj_id, contact_id):
        """
        Add a contact to the stage

        :param obj_id: int Stage ID
        :param contact_id: int Contact ID
        :return: dict|str
        """
        response = self._client.session.post(
            '{url}/{id}/contact/add/{contact_id}'.format(
                url=self.endpoint_url, id=obj_id, contact_id=contact_id
            )
        )
        return self.process_response(response)

    def remove_contact(self, obj_id, contact_id):
        """
        Remove a contact from the stage

        :param obj_id: int Stage ID
        :param contact_id: int Contact ID
        :return: dict|str
        """
        response = self._client.session.post(
            '{url}/{id}/contact/remove/{contact_id}'.format(
                url=self.endpoint_url, id=obj_id, contact_id=contact_id
            )
        )
        return self.process_response(response)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import

from .api import API


class Emails(API):
    _endpoint = 'emails'

    def send(self, obj_id):
        """
        Send email to the assigned lists

        :param obj_id: int
        :return: dict|str
        """
        response = self._client.session.post(
            '{url}/{id}/send'.format(
                url=self.endpoint_url, id=obj_id
            )
        )
        return self.process_response(response)

    def send_to_contact(self, obj_id, contact_id):
        """
        Send email to a specific contact

        :param obj_id: int
        :param contact_id: int
        :return: dict|str
        """
        response = self._client.session.post(
            '{url}/{id}/send/contact/{contact_id}'.format(
                url=self.endpoint_url, id=obj_id, contact_id=contact_id
            )
        )
        return self.process_response(response)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import

from .api import API


class Forms(API):
    _endpoint = 'forms'

    def delete_fields(self, form_id, field_ids):
        """
        Remove fields from a form

        :param form_id: int
        :param field_ids: list|tuple
        :return: dict|str
        """

        response = self._client.session.delete(
            '{url}/{form_id}/fields/delete'.format(
                url=self.endpoint_url, form_id=form_id
            ),
            params={'fields': field_ids}
        )
        return self.process_response(response)

    def delete_actions(self, form_id, action_ids):
        """
        Remove actions from a form

        :param form_id: int
        :param action_ids: list|tuple
        :return: dict|str
        """

        response = self._client.session.delete(
            '{url}/{form_id}/actions/delete'.format(
                url=self.endpoint_url, form_id=form_id
            ),
            params={'actions': action_ids}
        )
        return self.process_response(response)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import

from .api import API


class Files(API):
    _endpoint = 'files/images'

    def set_folder(self, folder='assets'):
        """
        Changes the file folder to look at

        :param folder: str [images, assets]
        :return: None
        """
        folder = folder.replace('/', '.')
        self._endpoint = 'files/{folder}'.format(folder=folder)

    def edit(self, obj_id, parameters=None, create_if_not_exists=False):
        return self.action_not_supported('edit')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import re

from setuptools import setup, find_packages


with open('mautic/__init__.py', 'r') as fd:
    version = re.search(
        r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]',
        fd.read(), re.MULTILINE
    ).group(1)

with open('README.rst', 'r') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst', 'r') as history_file:
    history = history_file.read()

requirements = [
    'requests-oauthlib',
]

test_requirements = []

setup(
    name='mautic',
    version=version,
    description='Python wrapper for Mautic API',
    long_description=readme + '\n\n' + history,
    author='Divio AG',
    author_email='info@divio.com',
    url='https://github.com/divio/python-mautic',
    packages=find_packages(),
    include_package_data=True,
    install_requires=requirements,
    license='LICENSE',
    zip_safe=False,
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
    test_suite='tests',
    tests_require=test_requirements
)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import

import requests
from requests_oauthlib import OAuth2Session

from .exceptions import MauticException


class MauticOauth2Client(object):
    def __init__(
        self,
        base_url,
        client_id,
        client_secret=None,
        scope=None,
        token=None,
        token_updater=None
    ):
        """
        :param base_url: str Base URL for Mautic API E.g. `https://<your-domain>.mautic.net`
        :param client_id: str Mautic API Public Key
        :param client_secret: str Mautic API Secret Key - needed to autorefresh token
        :param scope: list|str
        :param token: dict with access token data
        :param token_updater: function used for token autorefresh.
        """
        if scope and not isinstance(scope, (list, tuple)):
            scope = scope.split(',')
        self.base_url = base_url.strip(' /')

        self.access_token_url = base_url + '/oauth/v2/token'
        self.authorization_base_url = base_url + '/oauth/v2/authorize'

        if token_updater is not None and client_secret is not None:
            kwargs = {
                'auto_refresh_url': self.access_token_url,
                'auto_refresh_kwargs': {
                    'client_id': client_id,
                    'client_secret': client_secret
                },
                'token_updater': token_updater
            }
        else:
            kwargs = {}

        self.session = OAuth2Session(
            client_id, scope=scope, token=token, **kwargs
        )


class MauticBasicAuthClient(object):
    def __init__(
        self,
        base_url,
        username,
        password
    ):
        """
        :param base_url: str Base URL for Mautic API E.g. `https://<your-domain>.mautic.net`
        :param username: str Mautic Username
        :param password: str Mautic Password
        """

        self.base_url = base_url.strip(' /')

        self.session = requests.Session()
        self.session.auth = (username, password)


class API(object):
    _endpoint = ''

    def __init__(self, client):
        self._client = client
        self.endpoint_url = '{base_url}/api/{endpoint}'.format(
            base_url=self._client.base_url,
            endpoint=self._endpoint.strip(' /')
        )

    @staticmethod
    def process_response(response):
        if response.ok:
            return response.json()
        try:
            data = response.json()
        except ValueError:
            # no json object could be decoded
            data = response.content

        raise MauticException(data)

    @staticmethod
    def action_not_supported(action):
        """
        Returns a not supported error
        :param action: str
        :return: dict
        """
        return {
            'error': {
                'code': 500,
                'message':
                '{action} is not supported at this time'.format(action=action)
            }
        }

    def get(self, obj_id):
        """
        Get a single item

        :param obj_id: int
        :return: dict|str
        """
        response = self._client.session.get(
            '{url}/{id}'.format(
                url=self.endpoint_url, id=obj_id
            )
        )
        return self.process_response(response)

    def get_list(
        self,
        search='',
        start=0,
        limit=0,
        order_by='',
        order_by_dir='ASC',
        published_only=False,
        minimal=False
    ):
        """
        Get a list of items

        :param search: str
        :param start: int
        :param limit: int
        :param order_by: str
        :param order_by_dir: str
        :param published_only: bool
        :param minimal: bool
        :return: dict|str
        """

        parameters = {}
        args = ['search', 'start', 'limit', 'minimal']
        for arg in args:
            if arg in locals() and locals()[arg]:
                parameters[arg] = locals()[arg]
        if order_by:
            parameters['orderBy'] = order_by
        if order_by_dir:
            parameters['orderByDir'] = order_by_dir
        if published_only:
            parameters['publishedOnly'] = 'true'
        response = self._client.session.get(
            self.endpoint_url, params=parameters
        )
        return self.process_response(response)

    def get_published_list(
        self, search='', start=0, limit=0, order_by='', order_by_dir='ASC'
    ):
        """
        Proxy function to get_list with published_only set to True
        :param search: str
        :param start: int
        :param limit: int
        :param order_by: str
        :param order_by_dir: str
        :return: dict|str
        """
        return self.get_list(
            search=search,
            start=start,
            limit=limit,
            order_by=order_by,
            order_by_dir=order_by_dir,
            published_only=True
        )

    def create(self, parameters):
        """
        Create a new item (if supported)

        :param parameters: dict
        :return: dict|str
        """
        response = self._client.session.post(
            '{url}/new'.format(url=self.endpoint_url), data=parameters
        )
        return self.process_response(response)

    def edit(self, obj_id, parameters, create_if_not_exists=False):
        """
        Edit an item with option to create if it doesn't exist

        :param obj_id: int
        :param create_if_not_exists: bool
        :param parameters: dict
        :return: dict|str
        """
        if create_if_not_exists:
            response = self._client.session.put(
                '{url}/{id}/edit'.format(
                    url=self.endpoint_url, id=obj_id
                ),
                data=parameters
            )
        else:
            response = self._client.session.patch(
                '{url}/{id}/edit'.format(
                    url=self.endpoint_url, id=obj_id
                ),
                data=parameters
            )
        return self.process_response(response)

    def delete(self, obj_id):
        """
        Delete an item

        :param obj_id: int
        :return: dict|str
        """
        response = self._client.session.delete(
            '{url}/{id}/delete'.format(
                url=self.endpoint_url, id=obj_id
            )
        )
        return self.process_response(response)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import

from .api import API


class Segments(API):
    _endpoint = 'segments'

    def add_contact(self, segment_id, contact_id):
        """
        Add a contact to the segment

        :param segment_id: int Segment ID
        :param contact_id: int Contact ID
        :return: dict|str
        """

        response = self._client.session.post(
            '{url}/{segment_id}/contact/add/{contact_id}'.format(
                url=self.endpoint_url,
                segment_id=segment_id,
                contact_id=contact_id
            )
        )
        return self.process_response(response)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import

import json
import os
from tempfile import gettempdir

tmp = os.path.join(gettempdir(), 'mautic_creds.json')


def read_token_tempfile():
    """
    Example of function for getting stored token
    :return: token dict
    """
    with open(tmp, 'r') as f:
        return json.loads(f.read())


def update_token_tempfile(token):
    """
    Example of function for token update
    """
    with open(tmp, 'w') as f:
        f.write(json.dumps(token, indent=4))
